@@ -26,6 +26,8 @@
 #include <glib-object.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
+#include <libnotify/notification-hints.h>
+
 G_BEGIN_DECLS
 
 /**
@@ -94,9 +96,7 @@ typedef enum
  * @NOTIFY_CLOSED_REASON_DISMISSED: It has been dismissed by the user.
  * @NOTIFY_CLOSED_REASON_API_REQUEST: It has been closed by a call to
  *   [method@NotifyNotification.close].
- * @NOTIFY_CLOSED_REASON_UNDEFIEND: Closed by undefined/reserved reasons.
- *
- * The reason for which the notification has been closed.
+ * @NOTIFY_CLOSED_REASON_UNDEFINED: Closed by undefined/reserved reasons.
  *
  * Since: 0.8.0
  */
@@ -106,7 +106,11 @@ typedef enum
         NOTIFY_CLOSED_REASON_EXPIRED = 1,
         NOTIFY_CLOSED_REASON_DISMISSED = 2,
         NOTIFY_CLOSED_REASON_API_REQUEST = 3,
-        NOTIFY_CLOSED_REASON_UNDEFIEND = 4,
+        NOTIFY_CLOSED_REASON_UNDEFINED = 4,
+
+        NOTIFY_CLOSED_REASON_UNDEFIEND
+        __attribute__((__deprecated__(
+                "Use 'NOTIFY_CLOSED_REASON_UNDEFINED' instead"))) = 4,
 } NotifyClosedReason;
 
 /**
@@ -205,6 +209,8 @@ void                notify_notification_add_action            (NotifyNotificatio
 
 const char         *notify_notification_get_activation_token  (NotifyNotification *notification);
 
+GAppLaunchContext  *notify_notification_get_activation_app_launch_context (NotifyNotification *notification);
+
 void                notify_notification_clear_actions         (NotifyNotification *notification);
 gboolean            notify_notification_close                 (NotifyNotification *notification,
                                                                GError            **error);
